/* plot2ps, a utility for converting Unix plot files into postscript.
   Copyright (C) 1989 Free Software Foundation, Inc.

Plot2ps is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY.  No author or distributor accepts responsibility to anyone for the
consequences of using it or for whether it serves any particular purpose or
works at all, unless he says so in writing.  Refer to the GNU General Public
License for full details.

Everyone is granted permission to copy, modify and redistribute plot2ps, but
only under the conditions described in the GNU General Public License.  A copy
of this license is supposed to have been given to you along with plot2ps so
you can know your rights and responsibilities.  It should be in a file named
COPYING.  Among other things, the copyright notice and this notice must be
preserved on all copies.  */

/* This file is the main routine for plot2ps.

   It includes code to read the plot file and call plot functions
   to draw the graphics. */

#include <stdio.h>
#include <ctype.h>
#ifdef sequent
#include <strings.h>
#else
#include <string.h>
#endif
#include <math.h>
#include "plot3.h"

extern int perror();
extern int exit();
extern int printf();
extern int fprintf();
extern int fclose();
#ifndef mips
extern char *sprintf();
#endif
#ifndef hpux
extern void *malloc();
extern void *realloc();
#endif
extern int fputs();

enum plot_instruction
{
  ALABEL =	'T',
  ARC =		'a',
  CIRCLE =	'c',
  COLOR =	'C',
  CONT =	'n',
  ERASE =	'e',
  FILL =	'L',
  FONT =	'F',
  FONTSIZE =	'S',
  LABEL =	't',
  LINE =	'l',
  LINEMOD =	'f',
  MOVE =	'm',
  POINT =	'p',
  ROTATE =	'r',
  SPACE =	's',
};

/* This flag specifies that the input contains unsigned (if zero) or
   signed (if nonzero) two byte intgers. The default is signed. */
int signed_input = 1;

/* read in two byte intgers with high_byte_first. The default is to read the low
   byte first. */
int high_byte_first = 0;

/* guess_byte_order is a flag which, if non-zero, indicated that the function
   find_byte_order should be used to guess the byte order for the input file. */
int guess_byte_order = 1;

/* swap_bytes returns the bottom two bytes of its integer argument with
   their bytes reversed. */

int
swap_bytes (x)
     int x;
{
  unsigned char a, b;
  a = x & 0xff;
  b = (x >> 8) & 0xff;
  return ((a << 8)|b);
}

/* This is a set of known values for the maximum x (abscissa) values
   specified in the plot size command for known devices.  Using this
   set we can construct a heuristic proceedure for recognizing plot
   files in which the bytes of two byte integers are reversed.  We can
   recognize these files by looking for size commands containing these
   known sizes in byte reversed form. The last entry should be 0. */
   
int known_size[32] = {
  504,				/* plot3d output 504x504 */
  2048,				/* versatek plotter 2048x2048 */
  2100,				/* plot3d output */
  3120,				/* Tektronix 4014 terminal 3120x3120 */
  4096,				/* GSI 300 terminal 4096x4096 */
  0				/* the last entry should be 0 */
  };

/* find_byte_order takes four integer arguments and matches third one
   against a set of known values (sizes, see above). If there is a match
   it merely returns.  If there is no match, it check each of the values
   again with the bottom two bytes reversed. If such a match is found, the
   bottom two bytes of each argument is reversed, the high_byte_first flag
   is inverted to indicated how two byte integers should be read and the
   runction returns. */
   
void
find_byte_order (x0, y0, x1, y1)
     int *x0, *y0, *x1, *y1;
{
  int i;
  for (i=0; known_size[i]!=0; i++)
    {
      if (*x1 == known_size[i])
	return;
    }
  /* now check to see if reversing the bytes allows a match... */
  for (i=0; known_size[i]!=0; i++)
    {
      if (*x1 == swap_bytes (known_size[i]))
	{
	  *x0 = swap_bytes( *x0);
	  *y0 = swap_bytes( *y0);
	  *x1 = swap_bytes( *x1);
	  *y1 = swap_bytes( *y1);
	  high_byte_first = ! high_byte_first;
	  return;
	}
    }
}

/* Read a byte */

#define read_byte(stream) (getc (stream))

/* Read a coordinate - a two byte integer. */
  
int coord (input)
     FILE *input;
{
  int x;
  if ( high_byte_first )
    {
      x = ((char) read_byte (input)) << 8; /* get sign from high byte */
      x |= read_byte (input) & 0xFF; /* not from low byte */
    }
  else
    {
      x = read_byte (input) & 0xFF; /* ingnore sign in low byte */
      x |= ((char) read_byte (input)) << 8; /* get sign from high byte */
    }
  if ( ! signed_input )
    {
      x &= 0xFFFF;
    }

  return x;
}
  
/*  Read a string, change termination to null.
    note: string (buffer) reads a newline terminated string. */
  
void
read_string (input, buffer, buffer_length)
     FILE *input;
     char *buffer;
     int buffer_length;
{
  int length=0;
  char termination = '\n';
  char c = '\0';

  while (!feof (input))
    {
      if (length > buffer_length)
	{
	  buffer_length *= 2;
	  buffer = (char *) realloc (buffer, buffer_length);
	  if (buffer <= (char *) 0)
	    {
	      perror ("realloc failed:");
	      exit (-1);
	    }
	}
      c = read_byte (input);
      if (c == termination)
	break;
      buffer [length++] = c;
    }

  buffer [length] = '\0';	/*  null terminate label */
}


/* read_plot reads a plot file from the standard input and calls
   a plot function according to each plot instruction found in the
   file. */

void
read_plot (in_stream, buffer, buffer_length)
     FILE *in_stream;
     char *buffer;
     int  buffer_length;
{
  char x_adjust, y_adjust;
  int x0, y0, x1, y1, x2, y2, length, dx;
  int instruction;

  instruction = read_byte (in_stream);
  while (!feof (in_stream))
    {
      switch (instruction)
	{
	  /*  Note: we must get all but the last argument before calling to
	      ensure reading them in the proper order. */
	  
	case ALABEL:
	  x_adjust = read_byte (in_stream);
	  y_adjust = read_byte (in_stream); 
	  read_string (in_stream, buffer, buffer_length);
	  alabel (x_adjust, y_adjust, buffer);
	  break;
	case ARC:
	  x0 = coord (in_stream);
	  y0 = coord (in_stream);
	  x1 = coord (in_stream);
	  y1 = coord (in_stream); 
	  x2 = coord (in_stream);
	  y2 = coord (in_stream); 
	  arc (x0, y0, x1, y1, x2, y2);
	  break;
	case CIRCLE:
	  x0 = coord (in_stream);
	  y0 = coord (in_stream);
	  x1 = coord (in_stream);
	  circle (x0, y0, x1);
	  break;
	case COLOR:
	  x0 = coord (in_stream)&0xFFFF;
	  y0 = coord (in_stream)&0xFFFF;
	  x1 = coord (in_stream)&0xFFFF;
	  color (x0, y0, x1);
	  break;
	case CONT:
	  x0 = coord (in_stream);
	  y0 = coord (in_stream);
	  cont (x0, y0);
	  break;
	case ERASE:
	  erase (in_stream);
	  break;
	case FILL:
	  fill (coord (in_stream)&0xFFFF); break;
	case FONT:
	  read_string (in_stream, buffer, buffer_length);
	  fontname (buffer);
	  break;
	case FONTSIZE:
	  fontsize (coord (in_stream));
	  break;
	case LABEL:
	  read_string (in_stream, buffer, buffer_length);
	  label (buffer);
	  break;
	case LINE:
	  x0 = coord (in_stream);
	  y0 = coord (in_stream);
	  x1 = coord (in_stream);
	  y1 = coord (in_stream); 
	  line (x0, y0, x1, y1);
	  break;
	case LINEMOD:
	  read_string (in_stream, buffer, buffer_length);
	  linemod (buffer);
	  break;
	case MOVE:
	  x0 = coord (in_stream);
	  y0 = coord (in_stream);
	  move (x0, y0);
	  break;
	case POINT:
	  x0 = coord (in_stream);
	  y0 = coord (in_stream);
	  point (x0, y0);
	  break;
	case ROTATE:
	  x0 = coord (in_stream);
	  y0 = coord (in_stream);
	  x1 = coord (in_stream);
	  rotate (x0, y0, x1);
	  break;
	case SPACE:
	  x0 = coord (in_stream);
	  y0 = coord (in_stream);
	  x1 = coord (in_stream);
	  y1 = coord (in_stream); 
	  if (guess_byte_order)
	    find_byte_order(&x0, &y0, &x1, &y1);
	  space (x0, y0, x1, y1);
	  break;
	default:
	  fprintf (stderr, "Unrecognized plot command `%c' ignored.\n", instruction);
	}
      instruction = read_byte (in_stream);
    }
  return;
}


/* this is the copying notice (-copying option) */

char *copy_notice[] = {"\n\
		    GNU GENERAL PUBLIC LICENSE\n\
		     Version 1, February 1989\n\
\n\
 Copyright (C) 1989 Free Software Foundation, Inc.\n\
                    675 Mass Ave, Cambridge, MA 02139, USA\n\
 Everyone is permitted to copy and distribute verbatim copies\n\
 of this license document, but changing it is not allowed.\n\
\n\
			    Preamble\n\
\n\
  The license agreements of most software companies try to keep users\n\
at the mercy of those companies.  By contrast, our General Public\n\
","License is intended to guarantee your freedom to share and change free\n\
software--to make sure the software is free for all its users.  The\n\
General Public License applies to the Free Software Foundation's\n\
software and to any other program whose authors commit to using it.\n\
You can use it for your programs, too.\n\
\n\
  When we speak of free software, we are referring to freedom, not\n\
price.  Specifically, the General Public License is designed to make\n\
","sure that you have the freedom to give away or sell copies of free\n\
software, that you receive source code or can get it if you want it,\n\
that you can change the software or use pieces of it in new free\n\
programs; and that you know you can do these things.\n\
\n\
  To protect your rights, we need to make restrictions that forbid\n\
anyone to deny you these rights or to ask you to surrender the rights.\n\
These restrictions translate to certain responsibilities for you if you\n\
","distribute copies of the software, or if you modify it.\n\
\n\
  For example, if you distribute copies of a such a program, whether\n\
gratis or for a fee, you must give the recipients all the rights that\n\
you have.  You must make sure that they, too, receive or can get the\n\
source code.  And you must tell them their rights.\n\
\n\
  We protect your rights with two steps: (1) copyright the software, and\n\
(2) offer you this license which gives you legal permission to copy,\n\
","distribute and/or modify the software.\n\
\n\
  Also, for each author's protection and ours, we want to make certain\n\
that everyone understands that there is no warranty for this free\n\
software.  If the software is modified by someone else and passed on, we\n\
want its recipients to know that what they have is not the original, so\n\
that any problems introduced by others will not reflect on the original\n\
authors' reputations.\n\
\n\
","  The precise terms and conditions for copying, distribution and\n\
modification follow.\n\
\n\
		    GNU GENERAL PUBLIC LICENSE\n\
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION\n\
\n\
  0. This License Agreement applies to any program or other work which\n\
contains a notice placed by the copyright holder saying it may be\n\
distributed under the terms of this General Public License.  The\n\
\"Program\", below, refers to any such program or work, and a \"work based\n\
","on the Program\" means either the Program or any work containing the\n\
Program or a portion of it, either verbatim or with modifications.  Each\n\
licensee is addressed as \"you\".\n\
\n\
  1. You may copy and distribute verbatim copies of the Program's source\n\
code as you receive it, in any medium, provided that you conspicuously and\n\
appropriately publish on each copy an appropriate copyright notice and\n\
disclaimer of warranty; keep intact all the notices that refer to this\n\
","General Public License and to the absence of any warranty; and give any\n\
other recipients of the Program a copy of this General Public License\n\
along with the Program.  You may charge a fee for the physical act of\n\
transferring a copy.\n\
\n\
  2. You may modify your copy or copies of the Program or any portion of\n\
it, and copy and distribute such modifications under the terms of Paragraph\n\
1 above, provided that you also do the following:\n\
\n\
","    a) cause the modified files to carry prominent notices stating that\n\
    you changed the files and the date of any change; and\n\
\n\
    b) cause the whole of any work that you distribute or publish, that\n\
    in whole or in part contains the Program or any part thereof, either\n\
    with or without modifications, to be licensed at no charge to all\n\
    third parties under the terms of this General Public License (except\n\
    that you may choose to grant warranty protection to some or all\n\
","    third parties, at your option).\n\
\n\
    c) If the modified program normally reads commands interactively when\n\
    run, you must cause it, when started running for such interactive use\n\
    in the simplest and most usual way, to print or display an\n\
    announcement including an appropriate copyright notice and a notice\n\
    that there is no warranty (or else, saying that you provide a\n\
    warranty) and that users may redistribute the program under these\n\
","    conditions, and telling the user how to view a copy of this General\n\
    Public License.\n\
\n\
    d) You may charge a fee for the physical act of transferring a\n\
    copy, and you may at your option offer warranty protection in\n\
    exchange for a fee.\n\
\n\
Mere aggregation of another independent work with the Program (or its\n\
derivative) on a volume of a storage or distribution medium does not bring\n\
the other work under the scope of these terms.\n\
\n\
","  3. You may copy and distribute the Program (or a portion or derivative of\n\
it, under Paragraph 2) in object code or executable form under the terms of\n\
Paragraphs 1 and 2 above provided that you also do one of the following:\n\
\n\
    a) accompany it with the complete corresponding machine-readable\n\
    source code, which must be distributed under the terms of\n\
    Paragraphs 1 and 2 above; or,\n\
\n\
    b) accompany it with a written offer, valid for at least three\n\
","    years, to give any third party free (except for a nominal charge\n\
    for the cost of distribution) a complete machine-readable copy of the\n\
    corresponding source code, to be distributed under the terms of\n\
    Paragraphs 1 and 2 above; or,\n\
\n\
    c) accompany it with the information you received as to where the\n\
    corresponding source code may be obtained.  (This alternative is\n\
    allowed only for noncommercial distribution and only if you\n\
","    received the program in object code or executable form alone.)\n\
\n\
Source code for a work means the preferred form of the work for making\n\
modifications to it.  For an executable file, complete source code means\n\
all the source code for all modules it contains; but, as a special\n\
exception, it need not include source code for modules which are standard\n\
libraries that accompany the operating system on which the executable\n\
","file runs, or for standard header files or definitions files that\n\
accompany that operating system.\n\
\n\
  4. You may not copy, modify, sublicense, distribute or transfer the\n\
Program except as expressly provided under this General Public License.\n\
Any attempt otherwise to copy, modify, sublicense, distribute or transfer\n\
the Program is void, and will automatically terminate your rights to use\n\
the Program under this License.  However, parties who have received\n\
","copies, or rights to use copies, from you under this General Public\n\
License will not have their licenses terminated so long as such parties\n\
remain in full compliance.\n\
\n\
  5. By copying, distributing or modifying the Program (or any work based\n\
on the Program) you indicate your acceptance of this license to do so,\n\
and all its terms and conditions.\n\
\n\
  6. Each time you redistribute the Program (or any work based on the\n\
","Program), the recipient automatically receives a license from the original\n\
licensor to copy, distribute or modify the Program subject to these\n\
terms and conditions.  You may not impose any further restrictions on the\n\
recipients' exercise of the rights granted herein.\n\
\n\
  7. The Free Software Foundation may publish revised and/or new versions\n\
of the General Public License from time to time.  Such new versions will\n\
","be similar in spirit to the present version, but may differ in detail to\n\
address new problems or concerns.\n\
\n\
Each version is given a distinguishing version number.  If the Program\n\
specifies a version number of the license which applies to it and \"any\n\
later version\", you have the option of following the terms and conditions\n\
either of that version or of any later version published by the Free\n\
Software Foundation.  If the Program does not specify a version number of\n\
","the license, you may choose any version ever published by the Free Software\n\
Foundation.\n\
\n\
  8. If you wish to incorporate parts of the Program into other free\n\
programs whose distribution conditions are different, write to the author\n\
to ask for permission.  For software which is copyrighted by the Free\n\
Software Foundation, write to the Free Software Foundation; we sometimes\n\
make exceptions for this.  Our decision will be guided by the two goals\n\
","of preserving the free status of all derivatives of our free software and\n\
of promoting the sharing and reuse of software generally.\n\
\n\
			    NO WARRANTY\n\
\n\
  9. BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY\n\
FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN\n\
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES\n\
PROVIDE THE PROGRAM \"AS IS\" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED\n\
","OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF\n\
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS\n\
TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE\n\
PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,\n\
REPAIR OR CORRECTION.\n\
\n\
  10. IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING\n\
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR\n\
","REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,\n\
INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING\n\
OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED\n\
TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY\n\
YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER\n\
PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE\n\
POSSIBILITY OF SUCH DAMAGES.\n\
\n\
","		     END OF TERMS AND CONDITIONS\n\
\n\
	Appendix: How to Apply These Terms to Your New Programs\n\
\n\
  If you develop a new program, and you want it to be of the greatest\n\
possible use to humanity, the best way to achieve this is to make it\n\
free software which everyone can redistribute and change under these\n\
terms.\n\
\n\
  To do so, attach the following notices to the program.  It is safest to\n\
attach them to the start of each source file to most effectively convey\n\
","the exclusion of warranty; and each file should have at least the\n\
\"copyright\" line and a pointer to where the full notice is found.\n\
\n\
    <one line to give the program's name and a brief idea of what it does.>\n\
    Copyright (C) 19yy  <name of author>\n\
\n\
    This program is free software; you can redistribute it and/or modify\n\
    it under the terms of the GNU General Public License as published by\n\
    the Free Software Foundation; either version 1, or (at your option)\n\
","    any later version.\n\
\n\
    This program is distributed in the hope that it will be useful,\n\
    but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
    GNU General Public License for more details.\n\
\n\
    You should have received a copy of the GNU General Public License\n\
    along with this program; if not, write to the Free Software\n\
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\
\n\
","Also add information on how to contact you by electronic and paper mail.\n\
\n\
If the program is interactive, make it output a short notice like this\n\
when it starts in an interactive mode:\n\
\n\
    Gnomovision version 69, Copyright (C) 19xx name of author\n\
    Gnomovision comes with ABSOLUTELY NO WARRANTY; for details type `show w'.\n\
    This is free software, and you are welcome to redistribute it\n\
    under certain conditions; type `show c' for details.\n\
\n\
","The hypothetical commands `show w' and `show c' should show the\n\
appropriate parts of the General Public License.  Of course, the\n\
commands you use may be called something other than `show w' and `show\n\
c'; they could even be mouse-clicks or menu items--whatever suits your\n\
program.\n\
\n\
You should also get your employer (if you work as a programmer) or your\n\
school, if any, to sign a \"copyright disclaimer\" for the program, if\n\
necessary.  Here a sample; alter the names:\n\
\n\
","  Yoyodyne, Inc., hereby disclaims all copyright interest in the\n\
  program `Gnomovision' (a program to direct compilers to make passes\n\
  at assemblers) written by James Hacker.\n\
\n\
  <signature of Ty Coon>, 1 April 1989\n\
  Ty Coon, President of Vice\n\
\n\
That's all there is to it!\n\
",""};


int
main (argc, argv)
     int argc;
     char *argv[];
{
  int i;
  char *buffer;
  int  buffer_length;
  int  named_plot=0;		/* count named plot files on command line. */
  int  show_usage=0;		/* remember whether to output usage message. */
  int  opened=0;		/* remember whether we have opened output. */

  buffer_length = 1024;
  buffer = (char *) malloc (buffer_length);
  if (buffer <= (char *) 0)
    {
      perror ("malloc failed:");
      exit (-1);
    }

  for (i = 1; i < argc; i++)
    {
      if ((strcmp (argv [i], "-fontsize") == 0)
	  || (strcmp (argv [i], "-f") == 0))
	{
	  /* Sizes supported by X: 8, 10, 12, 14, 18, and 24. */
	  font_size = atof (argv[i+1]);
	  i += 1;
	}
      else if ((strcmp (argv [i], "-font") == 0)
	       || (strcmp (argv [i], "-fo") == 0))
	{
	  (void) fontname (argv [i+1]);
	  i += 1;
	}
      else if ((strcmp (argv [i], "-high-byte-first") == 0)
	       || (strcmp (argv [i], "-h") == 0))
	{
	  guess_byte_order = 0;
	  high_byte_first = 1;
	}
      else if ((strcmp (argv [i], "-low-byte-first") == 0)
	       || (strcmp (argv [i], "-l") == 0))
	{
	  guess_byte_order = 0;
	  high_byte_first = 0;
	}
      else if ((strcmp (argv [i], "-prologue") == 0)
	       || (strcmp (argv [i], "-p") == 0))
	{
	  user_has_prologue = 1;
	  users_prologue = argv[i+1];
	  i += 1;
	}
      else if ((strcmp (argv [i], "-warranty") == 0)
	       || (strcmp (argv [i], "-copying") == 0))
	{
	  for (i=0; copy_notice[i][0]>0; i++)
	    fputs (copy_notice[i], stdout);
	  named_plot++;
	}
      else if ((strcmp (argv [i], "-help") == 0)
	       || (strcmp (argv [i], "-V") == 0))
	{
	  show_usage++;
	  named_plot++;
	}
      else if ((strcmp (argv [i], "-signed") == 0))
	{
	  signed_input=1;
	}
      else if ((strcmp (argv [i], "-unsigned") == 0))
	{
	  signed_input=0;
	}
      else if ((strcmp (argv [i], "-") == 0))
	{
	  read_plot (stdin, buffer, buffer_length);
	  named_plot++;
	}
      else
	{
	  FILE *filep;
	  filep = fopen ( argv [i], "r");
	  if (filep == NULL)
	    {
	      fprintf (stderr, "Unrecognized option or file `%s' ignored.\n",
		       argv [i]);
	      show_usage++;
	    }
	  else
	    {
	      named_plot++;
	      if (!opened)
		{
		  openpl ();
		  opened ++;
		}
	      read_plot (filep, buffer, buffer_length);
	      fclose (filep);
	    }
	}
    }



  /* if there were no named plot files on the command line, then read
     the standard input. */
  if (named_plot == 0)
    {
      if (!opened)
	{
	  openpl ();
	  opened ++;
	}
      read_plot (stdin, buffer, buffer_length);
    }
  if (opened)
    closepl();

  if ( show_usage || !opened)
    {
      int font_index;

      fprintf (stderr, "\n\
usage: %s [-fontsize|-f SIZE] [-prologue|-p FILENAME]\n\
          [-high-byte-first|-h] [-low-byte-first|-l]\n\
          [-font|-fo FONTNAME] [-help|-V] [-copying|-warranty]\n\
          [PLOT_FILE_NAMES ...]\n\n\n",
	       *argv);
      fprintf (stderr, "Font names:\n");
      for (font_index=0; font_index<NO_OF_FONTS; font_index++)
	fprintf (stderr, "        %s\n", font_info[font_index].name);

      fprintf (stderr, "\n\n\
    plot2ps version 0.9, Copyright (C) 1989 Free Software Foundation, Inc.\n\
    plot2ps comes with ABSOLUTELY NO WARRANTY; type `plot2ps -warranty' for\n\
    details.  This is free software, and you are welcome to redistribute it;\n\
    Type `plot2ps -copying' to view the copying conditions.\n\n");
    }
  return 0;
}
